@@ -6,6 +6,7 @@ import logging
 import json
 import time
 from getpass import getuser
+from typing import Optional, Any
 
 from kafka import KafkaProducer
 from kafka.errors import NoBrokersAvailable
@@ -17,11 +18,12 @@ from firex_blaze.blaze_helper import BlazeSenderConfig, TASK_EVENT_TO_STATE, KAF
 
 logger = logging.getLogger(__name__)
 
-SEND_EVENT_TYPES = ('task-succeeded', 'task-failed', 'task-revoked', 'task-started-info', 'task-completed', 'task-results',
-                    'task-instrumentation')
+BLAZE_SEND_EVENT_TYPES = (
+    'task-succeeded', 'task-failed', 'task-revoked', 'task-started-info', 'task-completed', 'task-results',
+    'task-instrumentation')
 
 
-def format_kafka_message(firex_id, event_data, uuid, logs_url, submitter=getuser()):
+def format_kafka_message(firex_id, event_data, uuid, logs_url, submitter=getuser()) -> dict[str, Any]:
     return {'FIREX_ID': firex_id,
             'SUBMITTER': submitter,
             'LOGS_URL': logs_url,               # Shouldn't be required, but Lumens needs it!
@@ -29,7 +31,7 @@ def format_kafka_message(firex_id, event_data, uuid, logs_url, submitter=getuser
                         'UUID': uuid}]}
 
 
-def send_kafka_mssg(kafka_producer, kafka_mssg, kafka_topic, firex_id):
+def send_kafka_mssg(kafka_producer: KafkaProducer, kafka_mssg: dict[str, Any], kafka_topic: str, firex_id: str):
     kafka_producer.send(topic=kafka_topic,
                         value=json.dumps(kafka_mssg).encode('ascii'),
                         key=firex_id.encode('ascii'))
@@ -56,43 +58,38 @@ class NoNameForEvent(Exception):
 
 
 class KafkaSenderThread(BrokerEventConsumerThread):
-    """Captures Celery events and puts them on a Kafka bus."""
 
-    def __init__(self,
-                 celery_app,
-                 run_metadata: FireXRunMetadata,
-                 config: BlazeSenderConfig,
-                 logs_url: str,
-                 max_retry_attempts: int = None,
-                 receiver_ready_file: str = None,
-                 recording_file: str = None,
-                 max_kafka_connection_retries: int = 2):
+    def __init__(
+        self,
+        celery_app,
+        run_metadata: FireXRunMetadata,
+        config: BlazeSenderConfig,
+        max_retry_attempts: Optional[int]=None,
+        receiver_ready_file: Optional[str]=None,
+        recording_file: Optional[str]=None,
+    ):
 
         super().__init__(celery_app, max_retry_attempts, receiver_ready_file)
-        self.submitter = getuser()
         self.firex_id = run_metadata.firex_id
-        self.logs_url = logs_url
         self.kafka_topic = config.kafka_topic
         self.recording_file = recording_file
 
         # Connect to bootstrap servers and get a KafkaProducer instance
-        self.producer = self.get_kafka_producer(config, max_kafka_connection_retries)
-
-        self.uuid_to_task_name_mapping = {}
+        self.producer = self.get_kafka_producer(config)
         self.root_task = {'uuid': None, 'is_complete': False}
 
     @classmethod
-    def get_kafka_producer(cls, config, max_kafka_connection_retries):
+    def get_kafka_producer(cls, config: BlazeSenderConfig) -> KafkaProducer:
         _retries = 0
         while True:
             try:
                 return KafkaProducer(bootstrap_servers=config.kafka_bootstrap_servers)
             except NoBrokersAvailable as e:
-                if _retries < max_kafka_connection_retries:
+                if _retries < config.max_kafka_connection_retries:
                     _retries += 1
                     logger.exception(e)
                     logger.warning(f'Retrying connecting to boostrap servers '
-                                   f'[retry {_retries}/{max_kafka_connection_retries}]')
+                                   f'[retry {_retries}/{config.max_kafka_connection_retries}]')
                 else:
                     raise
 
@@ -100,15 +97,63 @@ class KafkaSenderThread(BrokerEventConsumerThread):
         return self.root_task['is_complete']
 
     def _update_root_task(self, event):
-        if (event.get('type') == 'task-received'
-                and 'root_id' in event
-                and self.root_task['uuid'] is None):
+        if (
+            event.get('type') == 'task-received'
+            and 'root_id' in event
+            and self.root_task['uuid'] is None
+        ):
             self.root_task['uuid'] = event['root_id']
 
         if event['uuid'] == self.root_task['uuid']:
             self.root_task['is_complete'] = event.get('type') in COMPLETE_RUNSTATES
 
-    def _get_kafka_event(self, event):
+    def _send_celery_event_to_kafka(self, event: dict[str, Any]) -> list[dict[str, Any]]:
+        raise NotImplementedError("Subclasses must implement sending.")
+
+    def _on_celery_event(self, event):
+        if 'uuid' not in event:
+            return
+
+        self._update_root_task(event)
+
+        sent_kafka_events = self._send_celery_event_to_kafka(event)
+
+        if sent_kafka_events and self.recording_file:
+            # Append the event to the recording file.
+            with open(self.recording_file, "a") as rec:
+                for e in sent_kafka_events:
+                    event_data_str = json.dumps(e, sort_keys=True, indent=2)
+                    rec.write(event_data_str + KAFKA_EVENTS_FILE_DELIMITER)
+
+    def _on_cleanup(self):
+        self.producer.flush()
+        self.producer.close()
+
+
+class BlazeKafkaSenderThread(KafkaSenderThread):
+    """Captures Celery events and puts them on a Kafka bus."""
+
+    def __init__(self,
+                 celery_app,
+                 run_metadata: FireXRunMetadata,
+                 config: BlazeSenderConfig,
+                 logs_url: str,
+                 max_retry_attempts: Optional[int] = None,
+                 receiver_ready_file: Optional[str] = None,
+                 recording_file: Optional[str] = None,
+    ):
+
+        super().__init__(
+            celery_app, run_metadata, config, max_retry_attempts,
+            receiver_ready_file, recording_file)
+
+        self.submitter = getuser()
+        self.firex_id = run_metadata.firex_id
+        self.logs_url = logs_url
+        self.kafka_topic = config.kafka_topic
+        self.uuid_to_task_name_mapping : dict[str, str] = {}
+
+    def _get_kafka_event(self, event: dict[str, Any]) -> dict[str, Any]:
         uuid = event.pop('uuid')
 
         if uuid not in self.uuid_to_task_name_mapping and 'long_name' in event:
@@ -118,7 +163,7 @@ class KafkaSenderThread(BrokerEventConsumerThread):
             name = self.uuid_to_task_name_mapping[uuid]
         else:
             # No need to produce this event since it won't be processed by Lumens anyways
-            raise NoNameForEvent(f'No name found for {event}; can not send the event')
+            raise NoNameForEvent(f'No task name found for {event}; can not send the event')
 
         # Remove result since we only should report firex_result, not the native result
         event.pop('result', None)
@@ -140,13 +185,8 @@ class KafkaSenderThread(BrokerEventConsumerThread):
                                     logs_url=self.logs_url,
                                     submitter=self.submitter)
 
-    def _on_celery_event(self, event):
-        if 'uuid' not in event:
-            return
-
-        self._update_root_task(event)
-
-        if event.get('type') in SEND_EVENT_TYPES:
+    def _send_celery_event_to_kafka(self, event: dict[str, Any]) -> list[dict[str, Any]]:
+        if event.get('type') in BLAZE_SEND_EVENT_TYPES:
             try:
                 kafka_event = self._get_kafka_event(event)
             except NoNameForEvent as e:
@@ -156,11 +196,6 @@ class KafkaSenderThread(BrokerEventConsumerThread):
                                 kafka_mssg=kafka_event,
                                 kafka_topic=self.kafka_topic,
                                 firex_id=self.firex_id)
+                return [kafka_event]
 
-                # Append the event to the recording file.
-                with open(self.recording_file, "a") as rec:
-                    rec.write(json.dumps(kafka_event, sort_keys=True, indent=2) + KAFKA_EVENTS_FILE_DELIMITER)
-
-    def _on_cleanup(self):
-        self.producer.flush()
-        self.producer.close()
+        return []
